import { mkdir, readFile, writeFile } from 'node:fs/promises';
import sharp from 'sharp';
import { createHash } from 'node:crypto';
import { readFileSync } from 'node:fs';
import { join } from 'node:path';
import baseN from 'base-n';

const defaultImageConfig = {
  w: [640, 750, 828, 1080, 1200, 1920, 2048, 3840],
  // TODO: remove this, needs fixing tests
  allowUpscale: "true",
  format: ["original", "webp"]
};
const queryArraySeparator = ";";
const META_KEY = "responsive-image";
function parseURL(id) {
  return new URL(id, "file://");
}
function getPathname(id) {
  const url = parseURL(id);
  return decodeURIComponent(url.pathname);
}
function parseQuery(query) {
  const params = query instanceof URLSearchParams ? query : new URLSearchParams(query);
  return Object.fromEntries(
    [...params.entries()].map(([key, value]) => {
      switch (key) {
        case "w":
          return [
            key,
            value.split(queryArraySeparator).map((v) => parseInt(v, 10))
          ];
        case "format":
          return [key, value.split(queryArraySeparator)];
        case "quality":
          return [key, parseInt(value, 10)];
        case "aspect":
          return [key, parseAspect(value)];
        case "lqip": {
          const parsedValue = value.charAt(0) === "{" ? JSON.parse(value) : value;
          return [
            key,
            typeof parsedValue === "string" ? { type: parsedValue } : parsedValue
          ];
        }
        default:
          return [key, value];
      }
    })
  );
}
function getOptions(id, defaultOptions, loaderOptions) {
  const url = id instanceof URL ? id : parseURL(id);
  const parsedResourceQuery = parseQuery(url.searchParams);
  return {
    ...defaultImageConfig,
    ...defaultOptions,
    ...loaderOptions,
    ...parsedResourceQuery
  };
}
function normalizeInput(input, options = {}) {
  if (typeof input === "string") {
    const filename = getPathname(input);
    input = readFileSync(filename);
  }
  if (Buffer.isBuffer(input)) {
    return {
      images: [],
      sharp: sharp(input),
      imports: [],
      hash: options.generateHash ? hash([input]) : void 0
    };
  }
  return input;
}
function parseAspect(aspect) {
  const parts = aspect.split(":");
  let aspectRatio;
  if (parts.length === 1) {
    aspectRatio = parseFloat(parts[0]);
  } else if (parts.length === 2) {
    const [width, height] = parts.map((str) => parseInt(str));
    if (!width || !height) return void 0;
    aspectRatio = width / height;
  }
  if (!aspectRatio || aspectRatio <= 0) return void 0;
  return aspectRatio;
}
function getAspectRatio(meta) {
  if (meta.height && meta.width && meta.height > 0) {
    return Math.round(meta.width / meta.height * 100) / 100;
  }
}
function onlyUnique(value, index, self) {
  return self.indexOf(value) === index;
}
function dataUri(data, type, base64 = false) {
  return `data:${type};base64,${base64 ? data : Buffer.from(data).toString("base64")}`;
}
function hash(datas) {
  const hash2 = createHash("md5");
  for (let data of datas) {
    if (typeof data === "object") {
      data = JSON.stringify(data);
    }
    hash2.update(data);
  }
  return hash2.digest("hex");
}
function getCacheFilename(input, config, extension, cacheDir) {
  if (!input.hash) {
    throw new Error("Expected hash to be availiable");
  }
  return join(cacheDir, `${hash([config, input.hash])}.${extension}`);
}

async function generateResizedImage(input, config, options = {}) {
  const { w, format } = config;
  if (typeof w !== "string") {
    throw new Error(
      "Expected width to be in image config. Looks like an internal bug in @responsive-image/vite-plugin."
    );
  }
  if (typeof format !== "string") {
    throw new Error(
      "Expected format to be in image config. Looks like an internal bug in @responsive-image/vite-plugin."
    );
  }
  const data = async () => {
    let cacheFile = void 0;
    if (options.cache && input.hash) {
      const cacheDir = options.cacheDir ?? "./node_modules/.cache";
      cacheFile = getCacheFilename(input, config, format, cacheDir);
      await mkdir(cacheDir, { recursive: true });
      try {
        const buffer = await readFile(cacheFile);
        return buffer;
      } catch (e) {
      }
    }
    const imagetools = await import('imagetools-core');
    const { transforms } = imagetools.generateTransforms(
      config,
      imagetools.builtins,
      new URLSearchParams()
    );
    const { image: resizedImage } = await imagetools.applyTransforms(
      transforms,
      input.sharp
    );
    const resizedImageData = await resizedImage.toBuffer();
    if (options.cache && cacheFile) {
      await writeFile(cacheFile, resizedImageData);
    }
    return resizedImageData;
  };
  return {
    data,
    width: parseInt(w, 10),
    format
  };
}
function effectiveImageFormats(formats, meta) {
  return formats.map((format) => {
    if (format === "original") {
      assertSupportedType(meta.format);
      return meta.format;
    } else {
      return format;
    }
  }).filter((value, index, self) => {
    return self.indexOf(value) === index;
  });
}
const supportedTypes = ["png", "jpeg", "webp", "avif"];
function assertSupportedType(type) {
  if (!supportedTypes.includes(type)) {
    throw new Error(`Unknown image type "${type}"`);
  }
}
async function getImagetoolsConfigs(options, buildKeys) {
  const imagetools = await import('imagetools-core');
  const entries = Object.entries(options).filter(([key]) => !buildKeys.includes(key)).map(([key, value]) => {
    const stringarrayifiedValue = Array.isArray(value) ? value.map((v) => String(v)) : [String(value)];
    return [key, stringarrayifiedValue];
  });
  return imagetools.resolveConfigs(entries, imagetools.builtinOutputFormats);
}

const generatedClassNames = /* @__PURE__ */ new Map();
const b64 = baseN.create();
function generateLqipClassName(resource) {
  if (generatedClassNames.has(resource)) {
    return generatedClassNames.get(resource);
  } else {
    const className = `ri-dyn-${b64.encode(generatedClassNames.size)}`;
    generatedClassNames.set(resource, className);
    return className;
  }
}
function blurrySvg(src, width, height) {
  return `<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" viewBox="0 0 ${width} ${height}">
<filter id="b" color-interpolation-filters="sRGB"><feGaussianBlur stdDeviation=".5"></feGaussianBlur><feComponentTransfer><feFuncA type="discrete" tableValues="1 1"></feFuncA></feComponentTransfer></filter>
<image filter="url(#b)" preserveAspectRatio="none" height="100%" width="100%" xlink:href="${src}"></image>
</svg>`;
}

export { META_KEY, blurrySvg, dataUri, effectiveImageFormats, generateLqipClassName, generateResizedImage, getAspectRatio, getCacheFilename, getImagetoolsConfigs, getOptions, getPathname, hash, normalizeInput, onlyUnique, parseAspect, parseQuery, parseURL };
